package java.nio;

import sun.misc.Cleaner;
import sun.misc.Unsafe;
import sun.nio.ch.DirectBuffer;

class DirectCharBufferU extends CharBuffer implements DirectBuffer
{
    protected static final Unsafe unsafe;
    private static final long arrayBaseOffset;
    protected static final boolean unaligned;
    private final Object att;
    
    @Override
    public Object attachment() {
        return this.att;
    }
    
    public Cleaner cleaner() {
        return null;
    }
    
    DirectCharBufferU(DirectBuffer att, int n, int n2, int n3, int n4, int n5) {
        super(n, n2, n3, n4);
        this.address = att.address() + (long)n5;
        this.att = att;
    }
    
    @Override
    public CharBuffer slice() {
        int position = this.position();
        int limit = this.limit();
        int n = (position <= limit) ? (limit - position) : 0;
        int n2 = position << 1;
        assert n2 >= 0;
        return new DirectCharBufferU(this, -1, 0, n, n, n2);
    }
    
    @Override
    public CharBuffer duplicate() {
        return new DirectCharBufferU(this, this.markValue(), this.position(), this.limit(), this.capacity(), 0);
    }
    
    @Override
    public CharBuffer asReadOnlyBuffer() {
        return new DirectCharBufferRU(this, this.markValue(), this.position(), this.limit(), this.capacity(), 0);
    }
    
    @Override
    public long address() {
        return this.address;
    }
    
    private long ix(int n) {
        return this.address + ((long)n << 1);
    }
    
    @Override
    public char get() {
        return DirectCharBufferU.unsafe.getChar(this.ix(this.nextGetIndex()));
    }
    
    @Override
    public char get(int n) {
        return DirectCharBufferU.unsafe.getChar(this.ix(this.checkIndex(n)));
    }
    
    @Override
    char getUnchecked(int n) {
        return DirectCharBufferU.unsafe.getChar(this.ix(n));
    }
    
    @Override
    public CharBuffer get(char[] array, int n, int n2) {
        if ((long)n2 << 1 > 6L) {
            Buffer.checkBounds(n, n2, array.length);
            int position = this.position();
            int limit = this.limit();
            assert position <= limit;
            if (n2 > ((position <= limit) ? (limit - position) : 0)) {
                throw new BufferUnderflowException();
            }
            if (this.order() != ByteOrder.nativeOrder()) {
                Bits.copyToCharArray(this.ix(position), array, (long)n << 1, (long)n2 << 1);
            }
            else {
                Bits.copyToArray(this.ix(position), array, DirectCharBufferU.arrayBaseOffset, (long)n << 1, (long)n2 << 1);
            }
            this.position(position + n2);
        }
        else {
            super.get(array, n, n2);
        }
        return this;
    }
    
    @Override
    public CharBuffer put(char x) {
        DirectCharBufferU.unsafe.putChar(this.ix(this.nextPutIndex()), x);
        return this;
    }
    
    @Override
    public CharBuffer put(int n, char x) {
        DirectCharBufferU.unsafe.putChar(this.ix(this.checkIndex(n)), x);
        return this;
    }
    
    @Override
    public CharBuffer put(CharBuffer charBuffer) {
        if ((charBuffer instanceof DirectCharBufferU)) {
            if (charBuffer == this) {
                throw new IllegalArgumentException();
            }
            DirectCharBufferU directCharBufferU = (DirectCharBufferU)charBuffer;
            int position2 = directCharBufferU.position();
            int limit2 = directCharBufferU.limit();
            assert position2 <= limit2;
            int n2 = (position2 <= limit2) ? (limit2 - position2) : 0;
            int position3 = this.position();
            int limit3 = this.limit();
            assert position3 <= limit3;
            if (n2 > ((position3 <= limit3) ? (limit3 - position3) : 0)) {
                throw new BufferOverflowException();
            }
            DirectCharBufferU.unsafe.copyMemory(directCharBufferU.ix(position2), this.ix(position3), (long)n2 << 1);
            directCharBufferU.position(position2 + n2);
            this.position(position3 + n2);
        }
        else if (charBuffer.hb != null) {
            int position = charBuffer.position();
            int limit = charBuffer.limit();
            assert position <= limit;
            int n = (position <= limit) ? (limit - position) : 0;
            this.put(charBuffer.hb, charBuffer.offset + position, n);
            charBuffer.position(position + n);
        }
        else {
            super.put(charBuffer);
        }
        return this;
    }
    
    @Override
    public CharBuffer put(char[] array, int n, int n2) {
        if ((long)n2 << 1 > 6L) {
            Buffer.checkBounds(n, n2, array.length);
            int position = this.position();
            int limit = this.limit();
            assert position <= limit;
            if (n2 > ((position <= limit) ? (limit - position) : 0)) {
                throw new BufferOverflowException();
            }
            if (this.order() != ByteOrder.nativeOrder()) {
                Bits.copyFromCharArray(array, (long)n << 1, this.ix(position), (long)n2 << 1);
            }
            else {
                Bits.copyFromArray(array, DirectCharBufferU.arrayBaseOffset, (long)n << 1, this.ix(position), (long)n2 << 1);
            }
            this.position(position + n2);
        }
        else {
            super.put(array, n, n2);
        }
        return this;
    }
    
    @Override
    public CharBuffer compact() {
        int position = this.position();
        int limit = this.limit();
        assert position <= limit;
        int n = (position <= limit) ? (limit - position) : 0;
        DirectCharBufferU.unsafe.copyMemory(this.ix(position), this.ix(0), (long)n << 1);
        this.position(n);
        this.limit(this.capacity());
        this.discardMark();
        return this;
    }
    
    @Override
    public boolean isDirect() {
        return true;
    }
    
    @Override
    public boolean isReadOnly() {
        return false;
    }
    
    public String toString(int n, int n2) {
        if (n2 > this.limit() || n > n2) {
            throw new IndexOutOfBoundsException();
        }
        try {
            char[] value = new char[n2 - n];
            CharBuffer wrap = CharBuffer.wrap(value);
            CharBuffer duplicate = this.duplicate();
            duplicate.position(n);
            duplicate.limit(n2);
            wrap.put(duplicate);
            return new String(value);
        }
        catch (StringIndexOutOfBoundsException ex) {
            throw new IndexOutOfBoundsException();
        }
    }
    
    @Override
    public CharBuffer subSequence(int n, int n2) {
        int position = this.position();
        int limit = this.limit();
        assert position <= limit;
        int n3 = (position <= limit) ? position : limit;
        int n4 = limit - n3;
        if (n < 0 || n2 > n4 || n > n2) {
            throw new IndexOutOfBoundsException();
        }
        return new DirectCharBufferU(this, -1, n3 + n, n3 + n2, this.capacity(), this.offset);
    }
    
    @Override
    public ByteOrder order() {
        return (ByteOrder.nativeOrder() != ByteOrder.BIG_ENDIAN) ? ByteOrder.LITTLE_ENDIAN : ByteOrder.BIG_ENDIAN;
    }
    
    static {
        unsafe = Bits.unsafe();
        arrayBaseOffset = (long)DirectCharBufferU.unsafe.arrayBaseOffset(char[].class);
        unaligned = Bits.unaligned();
    }
}
